#################################################################################################################
#Considering Over-dispersion in the Sample Size Calculation for Clinical Trials with Repeated Count Measurements#
#################################################################################################################
##########################################Sample Size for Comparing TAREquation 10##########################################
#1. m: number of measurements; 2. r: allocation ratio to control/treatment group; 3. ts: vector of time points (t0,t1,...,tm);
#4. beta1: rate of the control group on the log scale; 5. beta2: treatment effect; 6. theta: overdispersion parameter theta;
#7. corStructure: correlation structure either "ex": compound symmetric or "AR1": autoregressive of order 1;
#8. rho: correlation parameter; 9. delta: vector of missing probabilities at each time point;
#10. missingPattern: the missing pattern either "IM": independent missing or "MM": monotone missing;
#11. alpha: nominal level of type I error(e.g. 0.05); 12. power: nomial power(e.g. 0.8)
sampleSizeCal_TAR <- function(m,r,ts,beta1,beta2,theta,corStructure,rho,delta,missingPattern,alpha,power)
{
  Ts <- ts[2:(m+1)]-ts[1:m]
  mu1 <- exp(beta1)
  mu2 <- exp(beta1+beta2)
  sigma2sq.top <- 0
  sigma2sq.bottom <- 0
  for(j1 in 1:m)
  {
    for(j2 in 1:m)
    {
      if(corStructure == "ex")
      {
        if(j1 == j2)
        {
          rhoj1j2 <- 1
        }
        if(j1 != j2)
        {
          rhoj1j2 <- rho
        }  
      }
      if(corStructure == "AR1")
      {
        rhoj1j2 <- rho^abs(ts[2:(m+1)][j1]-ts[2:(m+1)][j2])
      }
      if(missingPattern == "IM")
      {
        deltaj1j2 <- prod(unique(c(delta[j1],delta[j2])))
      }
      if(missingPattern == "MM")
      {
        maxj <- max(j1,j2)
        deltaj1j2 <- delta[maxj]
      }
      sigma2sq.top <- sigma2sq.top + theta*deltaj1j2*rhoj1j2*sqrt(Ts[j1]*Ts[j2])
    }
  }
  for(j in 1:m)
  {
    sigma2sq.bottom <- sigma2sq.bottom + delta[j]*Ts[j]
  }
  sigma2sq.bottom <- sigma2sq.bottom^2
  sigma2sq <- sigma2sq.top/sigma2sq.bottom*(1/(1-r)/mu1 + 1/r/mu2)
  sampleSize <- ceiling((qnorm((1-alpha/2))+qnorm(power))^2*sigma2sq/beta2^2)
  return(sampleSize)
}
##########################################Sample Size for Comparing Slopes(Equation 15)##########################################
#1. m: number of measurements; 2. r: allocation ratio to control/treatment group; 3. ts: vector of time points (t0,t1,...,tm);
#4. beta1,beta2,beta3,beta4: coefficient parameters for mean; 5. theta: overdispersion parameter theta;
#6. corStructure: correlation structure either "ex": compound symmetric or "AR1": autoregressive of order 1;
#7. rho: correlation parameter; 8. delta: vector of missing probabilities at each time point;
#9. missingPattern: the missing pattern either "IM": independent missing or "MM": monotone missing;
#10. alpha: nominal level of type I error(e.g. 0.05); 11. power: nomial power(e.g. 0.8)
sampleSizeCal_Slope  <- function(m,r,ts,beta1,beta2,beta3,beta4,theta,corStructure,rho,delta,missingPattern,alpha,power)
{
  AC.matrix <- matrix(0,nrow=4,ncol=4)
  AT.matrix <- matrix(0,nrow=4,ncol=4)
  VC.matrix <- matrix(0,nrow=4,ncol=4)
  VT.matrix <- matrix(0,nrow=4,ncol=4)
  Ts <- ts[2:(m+1)]-ts[1:m]
  genMean.T <- function(period,duration)
  {
    coefficients <- c(beta1,beta2,beta3,beta4)
    zVector <- c(1,1,period,period)
    return(duration*exp(sum(coefficients * zVector)))
  }
  genMean.C <- function(period,duration)
  {
    coefficients <- c(beta1,beta2,beta3,beta4)
    zVector <- c(1,0,period,0)
    return(duration*exp(sum(coefficients * zVector)))
  }
  periods <- ts[-1]
  mu.T <- sapply(periods,function(x) genMean.T(x,Ts[match(x,periods)]))
  mu.C <- sapply(periods,function(x) genMean.C(x,Ts[match(x,periods)]))
  for(j1 in 1:m)
  {
    for(j2 in 1:m)
    {
      if(corStructure == "ex")
      {
        if(j1 == j2)
        {
          rhoj1j2 <- 1
        }
        if(j1 != j2)
        {
          rhoj1j2 <- rho
        }  
      }
      if(corStructure == "AR1")
      {
        rhoj1j2 <- rho^abs(periods[j1]-periods[j2])
      }
      if(missingPattern == "IM")
      {
        if(j1 == j2)
        {
          deltaj1j2 <- delta[j1]
        }
        if(j1 != j2)
        {
          deltaj1j2 <- delta[j1]*delta[j2]
        }
      }
      if(missingPattern == "MM")
      {
        maxj <- max(j1,j2)
        deltaj1j2 <- delta[maxj]
      }
      VC.matrix <-  VC.matrix + deltaj1j2*rhoj1j2*theta*sqrt(mu.C[j1]*mu.C[j2])*c(1,0,periods[j1],0)%*%t(c(1,0,periods[j2],0))
      VT.matrix <-  VT.matrix + deltaj1j2*rhoj1j2*theta*sqrt(mu.T[j1]*mu.T[j2])*c(1,1,periods[j1],periods[j1])%*%t(c(1,1,periods[j2],periods[j2]))
    }
  }
  for(j in 1:m)
  {
    AC.matrix <- AC.matrix + delta[j]*mu.C[j]*c(1,0,periods[j],0)%*%t(c(1,0,periods[j],0))
    AT.matrix <- AT.matrix + delta[j]*mu.T[j]*c(1,1,periods[j],periods[j])%*%t(c(1,1,periods[j],periods[j]))
  }
  A.matrix <- as.matrix((1-r)*AC.matrix + r*AT.matrix)
  V.matrix <- as.matrix((1-r)*VC.matrix + r*VT.matrix)
  sigma4sq <- (solve(A.matrix) %*% V.matrix %*% solve(A.matrix))[4,4]
  sampleSize <- ceiling((qnorm((1-alpha/2))+qnorm(power))^2*sigma4sq/beta4^2)
  return(sampleSize)
}